@@ -3,15 +3,12 @@ from .dtypes import DType
 
 class NodePortBP:
     """
-    The NodePorts (NodeInputs and NodeOutputs) are only placeholders for the static init_input and init_outputs of
-    custom Node classes.
+    The NodePortBPs (NodeInputs and NodeOutputs) are only placeholders (BluePrints) for the static init_input and
+    init_outputs of custom Node classes.
     An instantiated Node's actual inputs and outputs will be of type NodeObjPort (NodeObjInput, NodeObjOutput).
     """
 
     def __init__(self, label: str = '', type_: str = 'data'):
-        """
-        :type_: 'data' or 'exec'
-        """
 
         self.type_: str = type_
         self.label: str = label
@@ -19,7 +16,6 @@ class NodePortBP:
 
 class NodeInputBP(NodePortBP):
     def __init__(self, label: str = '', type_: str = 'data', dtype: DType = None, add_config={}):
-
         super().__init__(label, type_)
 
         self.dtype = dtype
